import os
import struct
import matplotlib.pyplot as plt
import scipy.io as scio
import math
import cmath
import numpy as np
from PIL import Image
# import warnings
# warnings.filterwarnings("ignore")


def sar_assign(sar_value):
    [hei, wid, cha] = sar_value.shape
    sar = np.zeros((hei, wid, cha), dtype=np.complex)
    sar[:, :, 0] = abs(sar_value[:, :, 0])
    sar[:, :, 1] = sar_value[:, :, 1] + 1j*sar_value[:, :, 2]
    sar[:, :, 2] = sar_value[:, :, 3] + 1j*sar_value[:, :, 4]
    sar[:, :, 2] = sar_value[:, :, 3] + 1j*sar_value[:, :, 4]
    sar[:, :, 3] = sar_value[:, :, 1] - 1j*sar_value[:, :, 2]
    sar[:, :, 4] = abs(sar_value[:, :, 5])
    sar[:, :, 5] = sar_value[:, :, 6] + 1j*sar_value[:, :, 7]
    sar[:, :, 6] = sar_value[:, :, 3] - 1j*sar_value[:, :, 4]
    sar[:, :, 7] = sar_value[:, :, 6] - 1j*sar_value[:, :, 7]
    sar[:, :, 8] = abs(sar_value[:, :, 8])
    return sar


def sar_extract(sar):
    [hei, wid, cha] = sar.shape
    sar_value = np.zeros((hei, wid, cha), dtype=np.float)
    sar_value[:, :, 0] = sar[:, :, 0]
    sar_value[:, :, 1] = sar[:, :, 1].real
    sar_value[:, :, 2] = sar[:, :, 1].imag
    sar_value[:, :, 3] = sar[:, :, 2].real
    sar_value[:, :, 4] = sar[:, :, 2].imag
    sar_value[:, :, 5] = sar[:, :, 4]
    sar_value[:, :, 6] = sar[:, :, 5].real
    sar_value[:, :, 7] = sar[:, :, 5].imag
    sar_value[:, :, 8] = sar[:, :, 8]
    return sar_value


def mat2pymat(mat):
    [hei, wid, cha] = mat.shape
    mat_py = np.zeros((cha, hei, wid), dtype=np.float)
    for i in range(hei):
        for j in range(wid):
            mat_py[:, i, j] = mat[i, j, :]
    return mat_py


def pymat2mat(mat_py):
    [cha, hei, wid] = mat_py.shape
    mat = np.zeros((hei, wid, cha), dtype=np.float)
    for i in range(hei):
        for j in range(wid):
            mat[i, j, :] = mat_py[:, i, j]
    return mat


def mat_reshape(mat_in, hei_out, wid_out):
    [num, cha, hei, wid] = mat_in.shape
    mat_out = np.zeros((hei_out, wid_out, cha), dtype=np.float)
    for i in range(int(wid_out/wid)):
        for j in range(int(hei_out/hei)):
            mat_out[j*hei:(j+1)*hei, i*wid:(i+1)*wid, :] = pymat2mat(mat_in[j*int(wid_out/wid)+i, :, :, :].reshape(cha, hei, wid))
    return mat_out


def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        pass
    try:
        import unicodedata
        unicodedata.numeric(s)
        return True
    except (TypeError, ValueError):
        pass
    return False


def C3_writebin(sar, savepath):
    str = '---------'
    datatype = 'C'
    sar_value = sar_extract(sar)
    [Nrow, Ncol, Ncha] = sar_value.shape
    savepath_new = os.path.join(savepath, '{}{}'.format(datatype, '3'))
    if not os.path.exists(savepath_new):
        os.mkdir(savepath_new)
    with open(os.path.join(savepath_new, 'config.txt'), 'w') as id0:
        id0.write('Nrow\n%d\n%s\nNcol\n%d\n%s\n' % (Nrow, str, Ncol, str))
        id0.write('PolarCase\nmonostatic\n%s\nPolarType\nfull\n' % (str))

    id1 = open(os.path.join(savepath_new, '{}{}'.format(datatype, '11.bin')), 'wb+')
    id2 = open(os.path.join(savepath_new, '{}{}'.format(datatype, '12_real.bin')), 'wb+')
    id3 = open(os.path.join(savepath_new, '{}{}'.format(datatype, '12_imag.bin')), 'wb+')
    id4 = open(os.path.join(savepath_new, '{}{}'.format(datatype, '13_real.bin')), 'wb+')
    id5 = open(os.path.join(savepath_new, '{}{}'.format(datatype, '13_imag.bin')), 'wb+')
    id6 = open(os.path.join(savepath_new, '{}{}'.format(datatype, '22.bin')), 'wb+')
    id7 = open(os.path.join(savepath_new, '{}{}'.format(datatype, '23_real.bin')), 'wb+')
    id8 = open(os.path.join(savepath_new, '{}{}'.format(datatype, '23_imag.bin')), 'wb+')
    id9 = open(os.path.join(savepath_new, '{}{}'.format(datatype, '33.bin')), 'wb+')

    for i in range(Nrow):
        for j in range(Ncol):
            id1.write(struct.pack('f', sar_value[i, j, 0]))
            id2.write(struct.pack('f', sar_value[i, j, 1]))
            id3.write(struct.pack('f', sar_value[i, j, 2]))
            id4.write(struct.pack('f', sar_value[i, j, 3]))
            id5.write(struct.pack('f', sar_value[i, j, 4]))
            id6.write(struct.pack('f', sar_value[i, j, 5]))
            id7.write(struct.pack('f', sar_value[i, j, 6]))
            id8.write(struct.pack('f', sar_value[i, j, 7]))
            id9.write(struct.pack('f', sar_value[i, j, 8]))










