import os
import warnings
import argparse
from psns20200622 import PSNS
from dataset import *
import glob
import re
from SARtoolbox import *
import h5py

warnings.filterwarnings("ignore")

parser = argparse.ArgumentParser(description='Pytorch PSFN')
parser.add_argument('--model', default='PSFN', type=str, help='choose path of model')
parser.add_argument('--batch_size', default=64, type=int, help='batch size')
parser.add_argument('--epoch', default=200, type=int, help='number of train epoch')
parser.add_argument('--lr', default=1e-4, type=float, help='initial learning rate for Adam')
parser.add_argument('--gpu', default='0', type=str, help='gpu id')
parser.add_argument('--hr_sar', default='data/test/sanfrancisco/hr_sar_sanfrancisco.mat', type=str, help='path of hr sar')
parser.add_argument('--lr_polsar', default='data/test/sanfrancisco/lr_polsar_sanfrancisco.mat', type=str, help='path of lr polsar')
parser.add_argument('--savepath', default='data/result/sanfrancisco/', type=str, help='path of lr polsar')
args = parser.parse_args()

cuda = torch.cuda.is_available()
os.environ["CUDA_VISIBLE_DEVICES"] = '0'
savedir = os.path.join('model', args.model)

if not os.path.exists(savedir) :
    os.mkdir(savedir)

if not os.path.exists(args.savepath):
    os.mkdir(args.savepath)


def data_loader(matpath):
    mat = h5py.File(matpath, 'r+')
    mat_in = np.transpose(mat[list(mat.keys())[0]])
    mat_in = np.array(mat_in).astype(np.float32)
    [num, cha, hei, wid] = mat_in.shape
    mat_in = torch.from_numpy(mat_in).contiguous().view(num, cha, hei, wid)
    mat_out = mat_in.float().cuda()
    return mat_out


def find_checkpoint(savedir):
    file_list = glob.glob(os.path.join(savedir, 'model_*.pth'))
    if file_list:
        epochs_exist = []
        for m in file_list:
            result = re.findall(".*model_(.*).pth.*", m)
            epochs_exist.append(int(result[0]))
        initial_epoch = max(epochs_exist)
    else :
        initial_epoch = 0
    return initial_epoch


def main():
    # load test data
    lr_polsar = data_loader(args.lr_polsar)
    hr_sar = data_loader(args.hr_sar)
    [num_lr, cha_lr, hei_lr, wid_lr] = lr_polsar.shape
    [num_hr, cha_hr, hei_hr, wid_hr] = hr_sar.shape
    hr_result = np.zeros((num_lr, cha_lr, hei_hr, wid_hr), dtype=np.float32)
    hr_result_np = np.zeros((num_lr, cha_lr, hei_hr-20, wid_hr-20), dtype=np.float32)

    model = PSNS()
    initial_epoch = find_checkpoint(savedir=savedir)
    if initial_epoch > 0:
        print('==> Resuming by loading epoch %03d' % initial_epoch)
        model = torch.load(os.path.join(savedir, 'model_%03d.pth' % initial_epoch))
    model.eval()
    if cuda:
        model = model.cuda()

    for n in range(num_hr):
        sub_lr_polsar = lr_polsar[n, :, :, :]
        sub_hr_sar = hr_sar[n, :, :, :]
        sub_lr_polsar = sub_lr_polsar.float().view(1, cha_lr, hei_lr, wid_lr)
        sub_hr_sar = sub_hr_sar.float().view(1, cha_hr, hei_hr, wid_hr)
        sub_lr_polsar = sub_lr_polsar.cuda()
        sub_hr_sar = sub_hr_sar.cuda()
        sub_out = model(sub_lr_polsar, sub_hr_sar)
        sub_out = sub_out.cpu()
        sub_out = sub_out.detach().numpy().astype(np.float32)
        hr_result[n, :, :, :] = sub_out

    hr_result_np = hr_result[:, :, 10: hei_hr-10, 10: wid_hr-10]
    hr_result_mat = mat_reshape(hr_result_np, 2400, 2400)
    hr = sar_assign(hr_result_mat)
    C3_writebin(hr, args.savepath)


if __name__ == '__main__':
    main()







